@@ -1,14 +1,26 @@
 use peace_performance::PpResult;
 use pyo3::{
-    prelude::{pyclass, pymethods},
+    prelude::{pyclass, pymethods, pyproto},
     types::PyDict,
-    PyResult, Python,
+    PyAny, PyCell, PyObjectProtocol, PyResult, Python,
 };
 
 use super::CalcResult;
 use crate::{methods::pp, objects::Beatmap, set_calculator};
 
-#[pyclass]
+macro_rules! generate_func {
+    ({$($attr:ident: $type:ty),*}) => {
+        paste::paste! {
+            $(
+                pub fn [<get_ $attr>](&self) -> $type { self.$attr }
+                pub fn [<set_ $attr>](&mut self, value: $type) { self.$attr = value; }
+                pub fn [<del_ $attr>](&mut self) { self.$attr = None; }
+            )*
+        }
+    };
+}
+
+#[pyclass(subclass)]
 #[derive(Debug, Default, Clone)]
 pub struct Calculator {
     #[pyo3(get, set)]
@@ -34,17 +46,37 @@ pub struct Calculator {
     #[pyo3(get, set)]
     pub score: Option<u32>,
 }
+crate::pyo3_py_protocol!(Calculator);
 
 #[pymethods]
 impl Calculator {
+    generate_func!({
+        mode: Option<u8>,
+        mods: Option<u32>,
+        n50: Option<usize>,
+        n100: Option<usize>,
+        n300: Option<usize>,
+        katu: Option<usize>,
+        acc: Option<f32>,
+        passed_obj: Option<usize>,
+        combo: Option<usize>,
+        miss: Option<usize>,
+        score: Option<u32>
+    });
+
     #[new]
-    pub fn new() -> Self {
-        Self::default()
+    #[args(data = "None", kwargs = "**")]
+    pub fn new(data: Option<&PyDict>, kwargs: Option<&PyDict>) -> PyResult<Self> {
+        let mut slf = Self::default();
+        if let Some(d) = data.or(kwargs) {
+            Self::set_with_dict(&mut slf, d)?;
+        }
+        Ok(slf)
     }
 
-    #[inline(always)]
-    pub fn calculate(&self, beatmap: &Beatmap) -> CalcResult {
-        CalcResult(self.calc(beatmap))
+    #[staticmethod]
+    pub fn new_empty() -> Self {
+        Self::default()
     }
 
     #[inline(always)]
@@ -62,11 +94,60 @@ impl Calculator {
         self.score = None;
     }
 
+    #[inline(always)]
+    pub fn calculate_raw(&self, beatmap: &Beatmap) -> CalcResult {
+        CalcResult(self.calc(beatmap))
+    }
+
+    pub fn getattr<'a>(slf: &'a PyCell<Self>, attr: &PyAny) -> PyResult<&'a PyAny> {
+        slf.as_ref().getattr(attr)
+    }
+
+    pub fn setattr<'a>(slf: &PyCell<Self>, attr: &PyAny, value: &PyAny) -> PyResult<()> {
+        slf.as_ref().setattr(attr, value)
+    }
+
+    #[inline(always)]
+    pub fn set_with_str(&mut self, attr: &str, value: &PyAny) -> PyResult<()> {
+        crate::set_with_py_str!(self, attr, value; {
+            mode,
+            mods,
+            n50,
+            n100,
+            n300,
+            katu,
+            acc,
+            passed_obj,
+            combo,
+            miss,
+            score
+        });
+        Ok(())
+    }
+
+    #[inline(always)]
+    pub fn set_with_dict(&mut self, data: &PyDict) -> PyResult<()> {
+        for (k, v) in data.iter() {
+            self.set_with_str(&k.extract::<String>()?, v)?;
+        }
+        Ok(())
+    }
+
     #[getter]
+    pub fn attrs(&self) -> String {
+        self.as_string()
+    }
+
+    #[getter]
+    pub fn attrs_dict<'a>(&self, py: Python<'a>) -> PyResult<&'a PyDict> {
+        self.as_dict(py)
+    }
+
+    #[getter]
+    #[inline(always)]
     pub fn as_string(&self) -> String {
         format!(
-            "mode: {:?}, mods: {:?}, n50: {:?}, n100: {:?}, n300: {:?}, katu: {:?}, 
-                acc: {:?}, passed_obj: {:?}, combo: {:?}, miss: {:?}, score: {:?}",
+            "mode: {:?}, mods: {:?}, n50: {:?}, n100: {:?}, n300: {:?}, katu: {:?}, acc: {:?}, passed_obj: {:?}, combo: {:?}, miss: {:?}, score: {:?}",
             self.mode,
             self.mods,
             self.n50,
@@ -82,6 +163,7 @@ impl Calculator {
     }
 
     #[getter]
+    #[inline(always)]
     pub fn as_dict<'a>(&self, py: Python<'a>) -> PyResult<&'a PyDict> {
         let d = crate::pyo3_py_dict!(py, self; {
             mode,
@@ -102,7 +184,7 @@ impl Calculator {
 
 impl Calculator {
     #[inline(always)]
-    #[timed::timed(duration(printer = "trace!"))]
+    #[cfg_attr(feature = "rust_logger", timed::timed(duration(printer = "trace!")))]
     pub fn calc(&self, beatmap: &Beatmap) -> PpResult {
         let c = pp::mode_any_pp(self.mode.unwrap_or(4), &beatmap.0);
         let c = set_calculator!(self.mods, c);
